package com.ruoyi.rc.mapper;

import java.math.BigDecimal;
import java.util.List;
import com.ruoyi.rc.domain.TAudit;

/**
 * 审计记录Mapper接口
 * 
 * @author ruoyi
 * @date 2024-07-19
 */
public interface TAuditMapper 
{
    public int selectHomeData1();

    public BigDecimal selectHomeData2();

    public int selectHomeData3(Long auditId);

    public int selectHomeData4(Long auditId);

    public Double selectHomeDataPie(Long deptId);

    public TAudit selectTAuditLatest(TAudit tAudit);

    /**
     * 查询审计记录
     * 
     * @param id 审计记录主键
     * @return 审计记录
     */
    public TAudit selectTAuditById(Long id);

    /**
     * 查询审计记录列表
     * 
     * @param tAudit 审计记录
     * @return 审计记录集合
     */
    public List<TAudit> selectTAuditList(TAudit tAudit);

    public List<TAudit> selectAllTAuditList(TAudit tAudit);

    /**
     * 新增审计记录
     * 
     * @param tAudit 审计记录
     * @return 结果
     */
    public int insertTAudit(TAudit tAudit);

    /**
     * 修改审计记录
     * 
     * @param tAudit 审计记录
     * @return 结果
     */
    public int updateTAudit(TAudit tAudit);

    /**
     * 删除审计记录
     * 
     * @param id 审计记录主键
     * @return 结果
     */
    public int deleteTAuditById(Long id);

    /**
     * 批量删除审计记录
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTAuditByIds(Long[] ids);
}
